Class: KamalBackup::CLI
- Inherits:
-
Thor
- Object
- Thor
- KamalBackup::CLI
show all
- Includes:
- Helpers
- Defined in:
- lib/kamal_backup/cli.rb
Defined Under Namespace
Modules: Helpers
Classes: CommandBase, DrillCLI, RestoreCLI
Class Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Methods included from Helpers
#accessory_name, #accessory_reboot_command, #bridge, #command_env, #confirm!, #confirm_production_restore!, #default_deploy_config?, #deploy_snippet, #deployment_mode?, #direct_app, #ensure_remote_version_match!, #exec_remote, #init_config_root, #local_command_config, #local_preferences, #local_restore_app, #print_backup_result, #print_prune_result, #print_remote_version_status, #production_restore_confirmation_config, #production_source_defaults, #prompt_required, #redactor, #remote_command_mode?, #remote_version, #require_typed_confirmation, #shared_config_path, #shared_config_source_defaults, #shared_config_template, #validate_deploy_config, #write_init_file
Class Attribute Details
.command_env ⇒ Object
Returns the value of attribute command_env.
387
388
389
|
# File 'lib/kamal_backup/cli.rb', line 387
def command_env
@command_env
end
|
Class Method Details
.basename ⇒ Object
425
426
427
|
# File 'lib/kamal_backup/cli.rb', line 425
def self.basename
'kamal-backup'
end
|
.normalize_global_options(argv) ⇒ Object
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
|
# File 'lib/kamal_backup/cli.rb', line 389
def normalize_global_options(argv)
tokens = Array(argv).dup
leading = []
while tokens.any?
token = tokens.first
case token
when '-d', '--destination', '-c', '--config-file'
leading << tokens.shift
leading << tokens.shift if tokens.any?
when /\A--destination=.+\z/, /\A--config-file=.+\z/
leading << tokens.shift
else
break
end
end
if leading.empty? || tokens.empty?
Array(argv)
else
[tokens.shift, *leading, *tokens]
end
end
|
.start(argv = ARGV, env: ENV) ⇒ Object
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
|
# File 'lib/kamal_backup/cli.rb', line 429
def self.start(argv = ARGV, env: ENV)
self.command_env = env
output = CommandOutput.new(io: $stderr, env: env)
Command.with_output(output) do
super(normalize_global_options(argv))
end
rescue Error => e
output ||= CommandOutput.new(io: $stderr, env: env)
output.error("(#{e.class}): #{e.message}", redactor: Redactor.new(env: env))
exit(1)
rescue StandardError => e
output ||= CommandOutput.new(io: $stderr, env: env)
output.error("(#{e.class}): #{e.message}", redactor: Redactor.new(env: env))
exit(1)
rescue Interrupt
output ||= CommandOutput.new(io: $stderr, env: env)
output.error('(Interrupt): interrupted', redactor: Redactor.new(env: env))
exit(130)
ensure
self.command_env = nil
end
|
Instance Method Details
#backup ⇒ Object
456
457
458
459
460
461
462
463
464
|
# File 'lib/kamal_backup/cli.rb', line 456
def backup
if remote_command_mode?
argv = %w[kamal-backup backup]
argv << '--force' if options[:force]
exec_remote(argv)
else
print_backup_result(direct_app.backup(force: options[:force]))
end
end
|
#check ⇒ Object
476
477
478
479
480
481
482
|
# File 'lib/kamal_backup/cli.rb', line 476
def check
if remote_command_mode?
exec_remote(%w[kamal-backup check])
else
puts(direct_app.check)
end
end
|
#evidence ⇒ Object
494
495
496
497
498
499
500
|
# File 'lib/kamal_backup/cli.rb', line 494
def evidence
if remote_command_mode?
exec_remote(%w[kamal-backup evidence])
else
puts(direct_app.evidence)
end
end
|
#init ⇒ Object
514
515
516
517
518
519
520
521
522
523
524
525
526
|
# File 'lib/kamal_backup/cli.rb', line 514
def init
write_init_file(shared_config_path, shared_config_template)
puts
puts 'Add this accessory block to your Kamal deploy config:'
puts
puts deploy_snippet
puts
puts 'The accessory runs scheduled database and file backups with backup.schedule.'
puts 'For most Rails apps, restore local and drill local can infer the development database, Active Storage path, and tmp state directory.'
puts 'Local restore and drill also require the restic binary on your machine.'
puts 'Create config/kamal-backup.local.yml only if you need to override those local defaults.'
end
|
#list ⇒ Object
467
468
469
470
471
472
473
|
# File 'lib/kamal_backup/cli.rb', line 467
def list
if remote_command_mode?
exec_remote(%w[kamal-backup list])
else
puts(direct_app.snapshots)
end
end
|
#prune ⇒ Object
485
486
487
488
489
490
491
|
# File 'lib/kamal_backup/cli.rb', line 485
def prune
if remote_command_mode?
exec_remote(%w[kamal-backup prune])
else
print_prune_result(direct_app.prune)
end
end
|
#schedule ⇒ Object
529
530
531
532
533
534
535
|
# File 'lib/kamal_backup/cli.rb', line 529
def schedule
if deployment_mode?
exec_remote(%w[kamal-backup schedule])
else
direct_app.schedule
end
end
|
#validate ⇒ Object
503
504
505
506
507
508
509
510
511
|
# File 'lib/kamal_backup/cli.rb', line 503
def validate
if remote_command_mode?
validate_deploy_config
else
direct_app.validate
end
puts('ok')
end
|
#version ⇒ Object
538
539
540
541
542
543
544
|
# File 'lib/kamal_backup/cli.rb', line 538
def version
if remote_command_mode?
print_remote_version_status
else
puts(VERSION)
end
end
|