Class: KamalBackup::CLI

Inherits:
Thor
  • Object
show all
Includes:
Helpers
Defined in:
lib/kamal_backup/cli.rb

Defined Under Namespace

Modules: Helpers Classes: CommandBase, DrillCLI, RestoreCLI

Class Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Helpers

#accessory_name, #accessory_reboot_command, #bridge, #command_env, #confirm!, #default_deploy_config?, #deploy_snippet, #deployment_mode?, #direct_app, #ensure_remote_version_match!, #exec_remote, #init_config_root, #local_app, #local_command_config, #local_preferences, #print_remote_version_status, #production_source_defaults, #prompt_required, #redactor, #remote_command_mode?, #remote_version, #shared_config_path, #shared_config_source_defaults, #shared_config_template, #validate_deploy_config, #write_init_file

Class Attribute Details

.command_envObject

Returns the value of attribute command_env.



302
303
304
# File 'lib/kamal_backup/cli.rb', line 302

def command_env
  @command_env
end

Class Method Details

.basenameObject



340
341
342
# File 'lib/kamal_backup/cli.rb', line 340

def self.basename
  "kamal-backup"
end

.normalize_global_options(argv) ⇒ Object



304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
# File 'lib/kamal_backup/cli.rb', line 304

def normalize_global_options(argv)
  tokens = Array(argv).dup
  leading = []

  while tokens.any?
    token = tokens.first

    case token
    when "-d", "--destination", "-c", "--config-file"
      leading << tokens.shift
      leading << tokens.shift if tokens.any?
    when /\A--destination=.+\z/, /\A--config-file=.+\z/
      leading << tokens.shift
    else
      break
    end
  end

  if leading.empty? || tokens.empty?
    Array(argv)
  else
    [tokens.shift, *leading, *tokens]
  end
end

.start(argv = ARGV, env: ENV) ⇒ Object



344
345
346
347
348
349
350
351
352
353
354
355
# File 'lib/kamal_backup/cli.rb', line 344

def self.start(argv = ARGV, env: ENV)
  self.command_env = env
  super(normalize_global_options(argv))
rescue Error => e
  warn("kamal-backup: #{Redactor.new(env: env).redact_string(e.message)}")
  exit(1)
rescue Interrupt
  warn("kamal-backup: interrupted")
  exit(130)
ensure
  self.command_env = nil
end

Instance Method Details

#backupObject



360
361
362
363
364
365
366
# File 'lib/kamal_backup/cli.rb', line 360

def backup
  if remote_command_mode?
    exec_remote(["kamal-backup", "backup"])
  else
    direct_app.backup
  end
end

#checkObject



378
379
380
381
382
383
384
# File 'lib/kamal_backup/cli.rb', line 378

def check
  if remote_command_mode?
    exec_remote(["kamal-backup", "check"])
  else
    puts(direct_app.check)
  end
end

#evidenceObject



387
388
389
390
391
392
393
# File 'lib/kamal_backup/cli.rb', line 387

def evidence
  if remote_command_mode?
    exec_remote(["kamal-backup", "evidence"])
  else
    puts(direct_app.evidence)
  end
end

#initObject



407
408
409
410
411
412
413
414
415
416
417
418
419
# File 'lib/kamal_backup/cli.rb', line 407

def init
  write_init_file(shared_config_path, shared_config_template)

  puts
  puts "Add this accessory block to your Kamal deploy config:"
  puts
  puts deploy_snippet
  puts
  puts "The accessory runs scheduled database and Active Storage backups with backup_schedule_seconds."
  puts "For most Rails apps, restore local and drill local can infer the development database, Active Storage path, and tmp state directory."
  puts "Local restore and drill also require the restic binary on your machine."
  puts "Create config/kamal-backup.local.yml only if you need to override those local defaults."
end

#listObject



369
370
371
372
373
374
375
# File 'lib/kamal_backup/cli.rb', line 369

def list
  if remote_command_mode?
    exec_remote(["kamal-backup", "list"])
  else
    puts(direct_app.snapshots)
  end
end

#scheduleObject



422
423
424
425
426
427
428
# File 'lib/kamal_backup/cli.rb', line 422

def schedule
  if deployment_mode?
    exec_remote(["kamal-backup", "schedule"])
  else
    direct_app.schedule
  end
end

#validateObject



396
397
398
399
400
401
402
403
404
# File 'lib/kamal_backup/cli.rb', line 396

def validate
  if remote_command_mode?
    validate_deploy_config
  else
    direct_app.validate
  end

  puts("ok")
end

#versionObject



431
432
433
434
435
436
437
# File 'lib/kamal_backup/cli.rb', line 431

def version
  if remote_command_mode?
    print_remote_version_status
  else
    puts(VERSION)
  end
end