Class: KamalBackup::CLI
- Inherits:
-
Thor
- Object
- Thor
- KamalBackup::CLI
show all
- Includes:
- Helpers
- Defined in:
- lib/kamal_backup/cli.rb
Defined Under Namespace
Modules: Helpers
Classes: CommandBase, DrillCLI, RestoreCLI
Class Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Methods included from Helpers
#accessory_name, #accessory_reboot_command, #bridge, #command_env, #confirm!, #default_deploy_config?, #deploy_snippet, #deployment_mode?, #direct_app, #ensure_remote_version_match!, #exec_remote, #init_config_root, #local_app, #local_command_config, #local_preferences, #print_remote_version_status, #prompt_required, #redactor, #remote_version, #shared_config_path, #shared_config_template, #version_remote_mode?, #write_init_file
Class Attribute Details
.command_env ⇒ Object
Returns the value of attribute command_env.
269
270
271
|
# File 'lib/kamal_backup/cli.rb', line 269
def command_env
@command_env
end
|
Class Method Details
.basename ⇒ Object
307
308
309
|
# File 'lib/kamal_backup/cli.rb', line 307
def self.basename
"kamal-backup"
end
|
.normalize_global_options(argv) ⇒ Object
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
|
# File 'lib/kamal_backup/cli.rb', line 271
def normalize_global_options(argv)
tokens = Array(argv).dup
leading = []
while tokens.any?
token = tokens.first
case token
when "-d", "--destination", "-c", "--config-file"
leading << tokens.shift
leading << tokens.shift if tokens.any?
when /\A--destination=.+\z/, /\A--config-file=.+\z/
leading << tokens.shift
else
break
end
end
if leading.empty? || tokens.empty?
Array(argv)
else
[tokens.shift, *leading, *tokens]
end
end
|
.start(argv = ARGV, env: ENV) ⇒ Object
311
312
313
314
315
316
317
318
319
320
321
322
|
# File 'lib/kamal_backup/cli.rb', line 311
def self.start(argv = ARGV, env: ENV)
self.command_env = env
super(normalize_global_options(argv))
rescue Error => e
warn("kamal-backup: #{Redactor.new(env: env).redact_string(e.message)}")
exit(1)
rescue Interrupt
warn("kamal-backup: interrupted")
exit(130)
ensure
self.command_env = nil
end
|
Instance Method Details
#backup ⇒ Object
327
328
329
330
331
332
333
|
# File 'lib/kamal_backup/cli.rb', line 327
def backup
if deployment_mode?
exec_remote(["kamal-backup", "backup"])
else
direct_app.backup
end
end
|
#check ⇒ Object
345
346
347
348
349
350
351
|
# File 'lib/kamal_backup/cli.rb', line 345
def check
if deployment_mode?
exec_remote(["kamal-backup", "check"])
else
puts(direct_app.check)
end
end
|
#evidence ⇒ Object
354
355
356
357
358
359
360
|
# File 'lib/kamal_backup/cli.rb', line 354
def evidence
if deployment_mode?
exec_remote(["kamal-backup", "evidence"])
else
puts(direct_app.evidence)
end
end
|
#init ⇒ Object
363
364
365
366
367
368
369
370
371
372
373
374
|
# File 'lib/kamal_backup/cli.rb', line 363
def init
write_init_file(shared_config_path, shared_config_template)
puts
puts "Add this accessory block to your Kamal deploy config:"
puts
puts deploy_snippet
puts
puts "For most Rails apps, restore local and drill local can infer the development database, storage path, and tmp state directory."
puts "Local restore and drill also require the restic binary on your machine."
puts "Create config/kamal-backup.local.yml only if you need to override those local defaults."
end
|
#list ⇒ Object
336
337
338
339
340
341
342
|
# File 'lib/kamal_backup/cli.rb', line 336
def list
if deployment_mode?
exec_remote(["kamal-backup", "list"])
else
puts(direct_app.snapshots)
end
end
|
#schedule ⇒ Object
377
378
379
380
381
382
383
|
# File 'lib/kamal_backup/cli.rb', line 377
def schedule
if deployment_mode?
exec_remote(["kamal-backup", "schedule"])
else
direct_app.schedule
end
end
|
#version ⇒ Object
386
387
388
389
390
391
392
|
# File 'lib/kamal_backup/cli.rb', line 386
def version
if version_remote_mode?
print_remote_version_status
else
puts(VERSION)
end
end
|