Class: KamalBackup::CLI
- Inherits:
-
Thor
- Object
- Thor
- KamalBackup::CLI
show all
- Includes:
- Helpers
- Defined in:
- lib/kamal_backup/cli.rb
Defined Under Namespace
Modules: Helpers
Classes: CommandBase, DrillCLI, RestoreCLI
Class Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Methods included from Helpers
#accessory_name, #accessory_reboot_command, #bridge, #command_env, #confirm!, #deploy_snippet, #deployment_mode?, #direct_app, #ensure_remote_version_match!, #exec_remote, #init_config_root, #local_app, #local_command_config, #local_preferences, #print_remote_version_status, #prompt_required, #redactor, #remote_version, #shared_config_path, #shared_config_template, #write_init_file
Class Attribute Details
.command_env ⇒ Object
Returns the value of attribute command_env.
261
262
263
|
# File 'lib/kamal_backup/cli.rb', line 261
def command_env
@command_env
end
|
Class Method Details
.basename ⇒ Object
299
300
301
|
# File 'lib/kamal_backup/cli.rb', line 299
def self.basename
"kamal-backup"
end
|
.normalize_global_options(argv) ⇒ Object
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
|
# File 'lib/kamal_backup/cli.rb', line 263
def normalize_global_options(argv)
tokens = Array(argv).dup
leading = []
while tokens.any?
token = tokens.first
case token
when "-d", "--destination", "-c", "--config-file"
leading << tokens.shift
leading << tokens.shift if tokens.any?
when /\A--destination=.+\z/, /\A--config-file=.+\z/
leading << tokens.shift
else
break
end
end
if leading.empty? || tokens.empty?
Array(argv)
else
[tokens.shift, *leading, *tokens]
end
end
|
.start(argv = ARGV, env: ENV) ⇒ Object
303
304
305
306
307
308
309
310
311
312
313
314
|
# File 'lib/kamal_backup/cli.rb', line 303
def self.start(argv = ARGV, env: ENV)
self.command_env = env
super(normalize_global_options(argv))
rescue Error => e
warn("kamal-backup: #{Redactor.new(env: env).redact_string(e.message)}")
exit(1)
rescue Interrupt
warn("kamal-backup: interrupted")
exit(130)
ensure
self.command_env = nil
end
|
Instance Method Details
#backup ⇒ Object
319
320
321
322
323
324
325
|
# File 'lib/kamal_backup/cli.rb', line 319
def backup
if deployment_mode?
exec_remote(["kamal-backup", "backup"])
else
direct_app.backup
end
end
|
#check ⇒ Object
337
338
339
340
341
342
343
|
# File 'lib/kamal_backup/cli.rb', line 337
def check
if deployment_mode?
exec_remote(["kamal-backup", "check"])
else
puts(direct_app.check)
end
end
|
#evidence ⇒ Object
346
347
348
349
350
351
352
|
# File 'lib/kamal_backup/cli.rb', line 346
def evidence
if deployment_mode?
exec_remote(["kamal-backup", "evidence"])
else
puts(direct_app.evidence)
end
end
|
#init ⇒ Object
355
356
357
358
359
360
361
362
363
364
365
366
|
# File 'lib/kamal_backup/cli.rb', line 355
def init
write_init_file(shared_config_path, shared_config_template)
puts
puts "Add this accessory block to your Kamal deploy config:"
puts
puts deploy_snippet
puts
puts "For most Rails apps, restore local and drill local can infer the development database, storage path, and tmp state directory."
puts "Local restore and drill also require the restic binary on your machine."
puts "Create config/kamal-backup.local.yml only if you need to override those local defaults."
end
|
#list ⇒ Object
328
329
330
331
332
333
334
|
# File 'lib/kamal_backup/cli.rb', line 328
def list
if deployment_mode?
exec_remote(["kamal-backup", "list"])
else
puts(direct_app.snapshots)
end
end
|
#schedule ⇒ Object
369
370
371
372
373
374
375
|
# File 'lib/kamal_backup/cli.rb', line 369
def schedule
if deployment_mode?
exec_remote(["kamal-backup", "schedule"])
else
direct_app.schedule
end
end
|
#version ⇒ Object
378
379
380
381
382
383
384
|
# File 'lib/kamal_backup/cli.rb', line 378
def version
if deployment_mode?
print_remote_version_status
else
puts(VERSION)
end
end
|