Class: KamalBackup::CLI

Inherits:
Thor
  • Object
show all
Includes:
Helpers
Defined in:
lib/kamal_backup/cli.rb

Defined Under Namespace

Modules: Helpers Classes: CommandBase, DrillCLI, RestoreCLI

Class Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Helpers

#accessory_name, #accessory_reboot_command, #bridge, #command_env, #confirm!, #confirm_production_restore!, #default_deploy_config?, #deploy_snippet, #deployment_mode?, #direct_app, #ensure_remote_version_match!, #exec_remote, #init_config_root, #local_command_config, #local_preferences, #local_restore_app, #print_remote_version_status, #production_restore_confirmation_config, #production_source_defaults, #prompt_required, #redactor, #remote_command_mode?, #remote_version, #require_typed_confirmation, #shared_config_path, #shared_config_source_defaults, #shared_config_template, #validate_deploy_config, #write_init_file

Class Attribute Details

.command_envObject

Returns the value of attribute command_env.



358
359
360
# File 'lib/kamal_backup/cli.rb', line 358

def command_env
  @command_env
end

Class Method Details

.basenameObject



396
397
398
# File 'lib/kamal_backup/cli.rb', line 396

def self.basename
  "kamal-backup"
end

.normalize_global_options(argv) ⇒ Object



360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
# File 'lib/kamal_backup/cli.rb', line 360

def normalize_global_options(argv)
  tokens = Array(argv).dup
  leading = []

  while tokens.any?
    token = tokens.first

    case token
    when "-d", "--destination", "-c", "--config-file"
      leading << tokens.shift
      leading << tokens.shift if tokens.any?
    when /\A--destination=.+\z/, /\A--config-file=.+\z/
      leading << tokens.shift
    else
      break
    end
  end

  if leading.empty? || tokens.empty?
    Array(argv)
  else
    [tokens.shift, *leading, *tokens]
  end
end

.start(argv = ARGV, env: ENV) ⇒ Object



400
401
402
403
404
405
406
407
408
409
410
411
412
413
# File 'lib/kamal_backup/cli.rb', line 400

def self.start(argv = ARGV, env: ENV)
  self.command_env = env
  Command.with_output(CommandOutput.new(io: $stderr)) do
    super(normalize_global_options(argv))
  end
rescue Error => e
  warn("kamal-backup: #{Redactor.new(env: env).redact_string(e.message)}")
  exit(1)
rescue Interrupt
  warn("kamal-backup: interrupted")
  exit(130)
ensure
  self.command_env = nil
end

Instance Method Details

#backupObject



418
419
420
421
422
423
424
# File 'lib/kamal_backup/cli.rb', line 418

def backup
  if remote_command_mode?
    exec_remote(["kamal-backup", "backup"])
  else
    direct_app.backup
  end
end

#checkObject



436
437
438
439
440
441
442
# File 'lib/kamal_backup/cli.rb', line 436

def check
  if remote_command_mode?
    exec_remote(["kamal-backup", "check"])
  else
    puts(direct_app.check)
  end
end

#evidenceObject



445
446
447
448
449
450
451
# File 'lib/kamal_backup/cli.rb', line 445

def evidence
  if remote_command_mode?
    exec_remote(["kamal-backup", "evidence"])
  else
    puts(direct_app.evidence)
  end
end

#initObject



465
466
467
468
469
470
471
472
473
474
475
476
477
# File 'lib/kamal_backup/cli.rb', line 465

def init
  write_init_file(shared_config_path, shared_config_template)

  puts
  puts "Add this accessory block to your Kamal deploy config:"
  puts
  puts deploy_snippet
  puts
  puts "The accessory runs scheduled database and file backups with backup.schedule."
  puts "For most Rails apps, restore local and drill local can infer the development database, Active Storage path, and tmp state directory."
  puts "Local restore and drill also require the restic binary on your machine."
  puts "Create config/kamal-backup.local.yml only if you need to override those local defaults."
end

#listObject



427
428
429
430
431
432
433
# File 'lib/kamal_backup/cli.rb', line 427

def list
  if remote_command_mode?
    exec_remote(["kamal-backup", "list"])
  else
    puts(direct_app.snapshots)
  end
end

#scheduleObject



480
481
482
483
484
485
486
# File 'lib/kamal_backup/cli.rb', line 480

def schedule
  if deployment_mode?
    exec_remote(["kamal-backup", "schedule"])
  else
    direct_app.schedule
  end
end

#validateObject



454
455
456
457
458
459
460
461
462
# File 'lib/kamal_backup/cli.rb', line 454

def validate
  if remote_command_mode?
    validate_deploy_config
  else
    direct_app.validate
  end

  puts("ok")
end

#versionObject



489
490
491
492
493
494
495
# File 'lib/kamal_backup/cli.rb', line 489

def version
  if remote_command_mode?
    print_remote_version_status
  else
    puts(VERSION)
  end
end