Class: KamalBackup::Config

Inherits:
Object
  • Object
show all
Defined in:
lib/kamal_backup/config.rb

Constant Summary collapse

DEFAULT_RETENTION =
{
  "RESTIC_KEEP_LAST" => "7",
  "RESTIC_KEEP_DAILY" => "7",
  "RESTIC_KEEP_WEEKLY" => "4",
  "RESTIC_KEEP_MONTHLY" => "6",
  "RESTIC_KEEP_YEARLY" => "2"
}.freeze
SUSPICIOUS_BACKUP_PATHS =
%w[/ /var /etc /root /usr /bin /sbin /boot /dev /proc /sys /run].freeze
SHARED_CONFIG_PATH =
"config/kamal-backup.yml"
LOCAL_CONFIG_PATH =
"config/kamal-backup.local.yml"
DEFAULT_CONFIG_PATHS =
[SHARED_CONFIG_PATH, LOCAL_CONFIG_PATH].freeze
YAML_KEY_ALIASES =
{
  "app_name" => "APP_NAME",
  "database_adapter" => "DATABASE_ADAPTER",
  "database_url" => "DATABASE_URL",
  "sqlite_database_path" => "SQLITE_DATABASE_PATH",
  "backup_paths" => "BACKUP_PATHS",
  "local_restore_source_paths" => "LOCAL_RESTORE_SOURCE_PATHS",
  "accessory" => "KAMAL_BACKUP_ACCESSORY",
  "restic_repository" => "RESTIC_REPOSITORY",
  "restic_repository_file" => "RESTIC_REPOSITORY_FILE",
  "restic_password" => "RESTIC_PASSWORD",
  "restic_password_file" => "RESTIC_PASSWORD_FILE",
  "restic_password_command" => "RESTIC_PASSWORD_COMMAND",
  "restic_init_if_missing" => "RESTIC_INIT_IF_MISSING",
  "restic_check_after_backup" => "RESTIC_CHECK_AFTER_BACKUP",
  "restic_check_read_data_subset" => "RESTIC_CHECK_READ_DATA_SUBSET",
  "restic_forget_after_backup" => "RESTIC_FORGET_AFTER_BACKUP",
  "restic_keep_last" => "RESTIC_KEEP_LAST",
  "restic_keep_daily" => "RESTIC_KEEP_DAILY",
  "restic_keep_weekly" => "RESTIC_KEEP_WEEKLY",
  "restic_keep_monthly" => "RESTIC_KEEP_MONTHLY",
  "restic_keep_yearly" => "RESTIC_KEEP_YEARLY",
  "backup_schedule_seconds" => "BACKUP_SCHEDULE_SECONDS",
  "backup_start_delay_seconds" => "BACKUP_START_DELAY_SECONDS",
  "state_dir" => "KAMAL_BACKUP_STATE_DIR",
  "allow_suspicious_paths" => "KAMAL_BACKUP_ALLOW_SUSPICIOUS_PATHS",
  "pgpassword" => "PGPASSWORD",
  "mysql_pwd" => "MYSQL_PWD"
}.freeze

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(env: ENV, cwd: Dir.pwd, defaults: {}, config_paths: nil, load_project_defaults: true) ⇒ Config

Returns a new instance of Config.



52
53
54
55
56
# File 'lib/kamal_backup/config.rb', line 52

def initialize(env: ENV, cwd: Dir.pwd, defaults: {}, config_paths: nil, load_project_defaults: true)
  raw_env = env.to_h
  base = load_project_defaults ? project_defaults(cwd: cwd) : {}
  @env = base.merge(defaults.to_h).merge(load_config_files(raw_env, cwd: cwd, paths: config_paths)).merge(raw_env)
end

Instance Attribute Details

#envObject (readonly)

Returns the value of attribute env.



50
51
52
# File 'lib/kamal_backup/config.rb', line 50

def env
  @env
end

Instance Method Details

#accessory_nameObject



66
67
68
# File 'lib/kamal_backup/config.rb', line 66

def accessory_name
  value("KAMAL_BACKUP_ACCESSORY")
end

#allow_in_place_file_restore?Boolean

Returns:

  • (Boolean)


106
107
108
# File 'lib/kamal_backup/config.rb', line 106

def allow_in_place_file_restore?
  truthy?("KAMAL_BACKUP_ALLOW_IN_PLACE_FILE_RESTORE")
end

#allow_suspicious_backup_paths?Boolean

Returns:

  • (Boolean)


110
111
112
# File 'lib/kamal_backup/config.rb', line 110

def allow_suspicious_backup_paths?
  truthy?("KAMAL_BACKUP_ALLOW_SUSPICIOUS_PATHS")
end

#app_nameObject



58
59
60
# File 'lib/kamal_backup/config.rb', line 58

def app_name
  value("APP_NAME")
end

#backup_path_label(path) ⇒ Object



157
158
159
160
# File 'lib/kamal_backup/config.rb', line 157

def backup_path_label(path)
  label = path.to_s.sub(%r{\A/+}, "").gsub(%r{[^A-Za-z0-9_.-]+}, "-")
  label.empty? ? "root" : label
end

#backup_pathsObject



134
135
136
# File 'lib/kamal_backup/config.rb', line 134

def backup_paths
  split_paths(value("BACKUP_PATHS"))
end

#backup_schedule_secondsObject



114
115
116
# File 'lib/kamal_backup/config.rb', line 114

def backup_schedule_seconds
  integer("BACKUP_SCHEDULE_SECONDS", 86_400, minimum: 1)
end

#backup_start_delay_secondsObject



118
119
120
# File 'lib/kamal_backup/config.rb', line 118

def backup_start_delay_seconds
  integer("BACKUP_START_DELAY_SECONDS", 0, minimum: 0)
end

#check_after_backup?Boolean

Returns:

  • (Boolean)


94
95
96
# File 'lib/kamal_backup/config.rb', line 94

def check_after_backup?
  truthy?("RESTIC_CHECK_AFTER_BACKUP")
end

#check_read_data_subsetObject



102
103
104
# File 'lib/kamal_backup/config.rb', line 102

def check_read_data_subset
  value("RESTIC_CHECK_READ_DATA_SUBSET")
end

#database_adapterObject



162
163
164
165
166
167
168
169
170
# File 'lib/kamal_backup/config.rb', line 162

def database_adapter
  if explicit = value("DATABASE_ADAPTER")
    normalize_adapter(explicit)
  elsif adapter = adapter_from_database_url
    adapter
  elsif value("SQLITE_DATABASE_PATH")
    "sqlite"
  end
end

#falsey?(key) ⇒ Boolean

Returns:

  • (Boolean)


295
296
297
# File 'lib/kamal_backup/config.rb', line 295

def falsey?(key)
  %w[0 false no n off].include?(value(key).to_s.downcase)
end

#forget_after_backup?Boolean

Returns:

  • (Boolean)


98
99
100
# File 'lib/kamal_backup/config.rb', line 98

def forget_after_backup?
  !falsey?("RESTIC_FORGET_AFTER_BACKUP")
end

#last_check_pathObject



126
127
128
# File 'lib/kamal_backup/config.rb', line 126

def last_check_path
  File.join(state_dir, "last_check.json")
end

#last_restore_drill_pathObject



130
131
132
# File 'lib/kamal_backup/config.rb', line 130

def last_restore_drill_path
  File.join(state_dir, "last_restore_drill.json")
end

#local_restore_path_pairsObject



146
147
148
149
150
151
152
153
154
155
# File 'lib/kamal_backup/config.rb', line 146

def local_restore_path_pairs
  source_paths = local_restore_source_paths
  target_paths = backup_paths

  if source_paths.size == target_paths.size
    source_paths.zip(target_paths)
  else
    raise ConfigurationError, "LOCAL_RESTORE_SOURCE_PATHS must contain the same number of paths as BACKUP_PATHS"
  end
end

#local_restore_source_pathsObject



138
139
140
141
142
143
144
# File 'lib/kamal_backup/config.rb', line 138

def local_restore_source_paths
  if raw = value("LOCAL_RESTORE_SOURCE_PATHS")
    split_paths(raw)
  else
    backup_paths
  end
end

#production_like_target?(target) ⇒ Boolean

Returns:

  • (Boolean)


269
270
271
272
273
274
275
276
277
# File 'lib/kamal_backup/config.rb', line 269

def production_like_target?(target)
  target = target.to_s

  if source_database_targets.include?(target)
    true
  else
    production_named_target?(target.downcase)
  end
end

#required_app_nameObject



62
63
64
# File 'lib/kamal_backup/config.rb', line 62

def required_app_name
  required_value("APP_NAME")
end

#required_value(key) ⇒ Object



287
288
289
# File 'lib/kamal_backup/config.rb', line 287

def required_value(key)
  value(key) || raise(ConfigurationError, "#{key} is required")
end

#restic_init_if_missing?Boolean

Returns:

  • (Boolean)


90
91
92
# File 'lib/kamal_backup/config.rb', line 90

def restic_init_if_missing?
  truthy?("RESTIC_INIT_IF_MISSING")
end

#restic_passwordObject



78
79
80
# File 'lib/kamal_backup/config.rb', line 78

def restic_password
  value("RESTIC_PASSWORD")
end

#restic_password_commandObject



86
87
88
# File 'lib/kamal_backup/config.rb', line 86

def restic_password_command
  value("RESTIC_PASSWORD_COMMAND")
end

#restic_password_fileObject



82
83
84
# File 'lib/kamal_backup/config.rb', line 82

def restic_password_file
  value("RESTIC_PASSWORD_FILE")
end

#restic_repositoryObject



70
71
72
# File 'lib/kamal_backup/config.rb', line 70

def restic_repository
  value("RESTIC_REPOSITORY")
end

#restic_repository_fileObject



74
75
76
# File 'lib/kamal_backup/config.rb', line 74

def restic_repository_file
  value("RESTIC_REPOSITORY_FILE")
end

#retentionObject



172
173
174
175
176
# File 'lib/kamal_backup/config.rb', line 172

def retention
  DEFAULT_RETENTION.each_with_object({}) do |(key, default), result|
    result[key] = value(key) || default
  end
end

#retention_argsObject



178
179
180
181
182
183
184
185
186
187
188
189
190
# File 'lib/kamal_backup/config.rb', line 178

def retention_args
  retention.each_with_object([]) do |(key, raw), args|
    next if raw.to_s.empty?

    number = Integer(raw)
    next if number <= 0

    flag = "--#{key.sub("RESTIC_KEEP_", "keep-").downcase.tr("_", "-")}"
    args.concat([flag, number.to_s])
  rescue ArgumentError
    raise ConfigurationError, "#{key} must be an integer"
  end
end

#state_dirObject



122
123
124
# File 'lib/kamal_backup/config.rb', line 122

def state_dir
  value("KAMAL_BACKUP_STATE_DIR") || "/var/lib/kamal-backup"
end

#truthy?(key) ⇒ Boolean

Returns:

  • (Boolean)


291
292
293
# File 'lib/kamal_backup/config.rb', line 291

def truthy?(key)
  %w[1 true yes y on].include?(value(key).to_s.downcase)
end

#validate_backup(check_files: true) ⇒ Object



198
199
200
201
202
# File 'lib/kamal_backup/config.rb', line 198

def validate_backup(check_files: true)
  validate_restic(check_files: check_files)
  validate_database_backup(check_files: check_files)
  validate_backup_paths(check_files: check_files)
end

#validate_backup_paths(check_files: true) ⇒ Object

Raises:



227
228
229
230
231
232
233
234
235
236
237
238
# File 'lib/kamal_backup/config.rb', line 227

def validate_backup_paths(check_files: true)
  paths = backup_paths
  raise ConfigurationError, "BACKUP_PATHS must contain at least one path" if paths.empty?

  paths.each do |path|
    expanded = File.expand_path(path)
    if SUSPICIOUS_BACKUP_PATHS.include?(expanded) && !allow_suspicious_backup_paths?
      raise ConfigurationError, "refusing suspicious backup path #{expanded}; set KAMAL_BACKUP_ALLOW_SUSPICIOUS_PATHS=true to override"
    end
    raise ConfigurationError, "backup path does not exist: #{path}" if check_files && !File.exist?(path)
  end
end

#validate_database_backup(check_files: true) ⇒ Object



209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
# File 'lib/kamal_backup/config.rb', line 209

def validate_database_backup(check_files: true)
  case database_adapter
  when "postgres"
    unless value("DATABASE_URL") || value("PGDATABASE")
      raise ConfigurationError, "PostgreSQL backup requires DATABASE_URL or PGDATABASE/libpq environment"
    end
  when "mysql"
    unless value("DATABASE_URL") || value("MYSQL_DATABASE") || value("MARIADB_DATABASE")
      raise ConfigurationError, "MySQL backup requires DATABASE_URL or MYSQL_DATABASE/MARIADB_DATABASE"
    end
  when "sqlite"
    path = required_value("SQLITE_DATABASE_PATH")
    raise ConfigurationError, "SQLITE_DATABASE_PATH does not exist: #{path}" if check_files && !File.file?(path)
  else
    raise ConfigurationError, "DATABASE_ADAPTER is required or must be detectable from DATABASE_URL/SQLITE_DATABASE_PATH"
  end
end

#validate_database_restore_target(target) ⇒ Object

Raises:



261
262
263
264
265
266
267
# File 'lib/kamal_backup/config.rb', line 261

def validate_database_restore_target(target)
  raise ConfigurationError, "restore database target is required" if target.to_s.strip.empty?

  if production_like_target?(target)
    raise ConfigurationError, "refusing production-looking restore target #{target}; choose a scratch target that does not look like production"
  end
end

#validate_file_restore_target(target) ⇒ Object

Raises:



248
249
250
251
252
253
254
255
256
257
258
259
# File 'lib/kamal_backup/config.rb', line 248

def validate_file_restore_target(target)
  raise ConfigurationError, "restore target cannot be empty" if target.to_s.strip.empty?

  expanded_target = File.expand_path(target)
  raise ConfigurationError, "refusing to restore files to /" if expanded_target == "/"

  if in_place_file_restore?(expanded_target) && !allow_in_place_file_restore?
    raise ConfigurationError, "refusing in-place file restore to #{expanded_target}; set KAMAL_BACKUP_ALLOW_IN_PLACE_FILE_RESTORE=true to override"
  end

  expanded_target
end

#validate_local_database_restore_target(target) ⇒ Object

Raises:



240
241
242
243
244
245
246
# File 'lib/kamal_backup/config.rb', line 240

def validate_local_database_restore_target(target)
  raise ConfigurationError, "local restore database target is required" if target.to_s.strip.empty?

  if production_named_target?(target)
    raise ConfigurationError, "refusing production-looking local restore target #{target}; use restore production for production restores"
  end
end

#validate_local_machine_restoreObject



204
205
206
207
# File 'lib/kamal_backup/config.rb', line 204

def validate_local_machine_restore
  validate_local_machine_environment
  validate_local_machine_paths
end

#validate_restic(check_files: true) ⇒ Object



192
193
194
195
196
# File 'lib/kamal_backup/config.rb', line 192

def validate_restic(check_files: true)
  required_app_name
  validate_restic_repository(check_files: check_files)
  validate_restic_password(check_files: check_files)
end

#value(key) ⇒ Object



279
280
281
282
283
284
285
# File 'lib/kamal_backup/config.rb', line 279

def value(key)
  raw = env[key]
  return nil if raw.nil?

  stripped = raw.to_s.strip
  stripped.empty? ? nil : stripped
end